% calculates intermediate variables

%  ============================================================================
%    CASY_ECMP.C  - evaluate computed parameters
%
%      Includes:
%	EVAL_CMP_BLADEORIGIN
%	EVAL_CMP_BLADECIRCLE
%	EVAL_CMP_BLADECIRCLERAD
%	EVAL_CMP_TONGBLADE
%	EVAL_CMP_FRONTFACTOR
%	EVAL_CMP_FLOOR2TEETH
%	EVAL_CMP_TIPCIRCLE
%	EVAL_CMP_TIPTRACKING
%	EVAL_CMP_ROOT
%	EVAL_CMP_HYOIDTONGTAN
%	EVAL_CMP_TONGUECIRCLE
%  ============================================================================




%  ============================================================================
%    EVAL_CMP_BLADEORIGIN (B):  origin of tongue tip segment on tongue circle:
%				the actual point of departure from the tongue
%    ball surface (i.e. VAR_TongBlade or B') rotates around this point as 
%    a function of tongue tip height.  CMP_BladeOrigin is the intersection
%    of a line drawn from the tongue ball center (CMP_TongueCircle) to its
%    circumference at an angle of KEY_Jaw.polar.ang (THJ) plus 
%    GEO_BladeOrigin.polar.ang (.55pi) plus KEY_BladeOriginOffset.polar.ang 
%    with respect to the horizontal.
%  ============================================================================


% void eval_cmp_BladeOrigin()
% {
%   double a;
% 
%   a = KEY_Jaw.polar.ang + GEO_BladeOrigin.polar.ang + 
%       KEY_BladeOriginOffset.polar.ang;
%   CMP_BladeOrigin.coord.x = CMP_TongueCircle.coord.x + 
%                       KEY_TongueCircleRad.polar.len * cos( a );
%   CMP_BladeOrigin.coord.y = CMP_TongueCircle.coord.y + 
%                       KEY_TongueCircleRad.polar.len * sin( a );
% 
% }

a = KEY_Jaw(4) + GEO_BladeOrigin(4) + KEY_BladeOriginOffset(4)
CMP_BladeOrigin = CMP_TongueCircle + KEY_TongueCircleRad(3) * [ cos( a ) sin( a ) ]



%  ============================================================================
%    EVAL_CMP_BLADECIRCLE:  center of tongue blade circle; lies along the 
%				extension of the line determined by the 
%    tongue ball center (CMP_TongueCircle, C) and the point the outline curve 
%    diverges from the tongue ball circle (VAR_TongBlade, B').  The slope of
%    this line is described by CMP_TongBlade.polar.ang
%
%  ============================================================================


% void eval_cmp_BladeCircle()
% {
%   CMP_BladeCircle.coord.x = VAR_TongBlade.coord.x + 
%                CMP_BladeCircleRad.polar.len * cos(CMP_TongBlade.polar.ang);
%   CMP_BladeCircle.coord.y = VAR_TongBlade.coord.y + 
%                CMP_BladeCircleRad.polar.len * sin(CMP_TongBlade.polar.ang);
% 
% }

CMP_BladeCircle = VAR_TongBlade + ...
    CMP_BladeCircleRad(3) * [ cos(CMP_TongBlade(4)) sin(CMP_TongBlade(4)) ]


%  ============================================================================
%    EVAL_CMP_BLADECIRCLERAD:  radius of tongue blade circle
%
%    The tongue blade circle describes the curvature of the variable outline
%    between the tongue ball and the tongue tip circles.  It has its center 
%    (CMP_BladeCircle, Q) on the extension of the line determined by the 
%    tongue ball center (CMP_TongueCircle, C) and the point the outline curve 
%    diverges from the tongue ball circle (VAR_TongBlade, B').  It is tangent 
%    to the tongue ball circle at point VAR_TongBlade, B', and to the tip 
%    circle at VAR_BladeTip.  By the Rule of Cosines
%
%	(BQ+Tr)^2 = BQ^2 + BT^2 - 2 BQ BT cos(theta)
%
%    where Tr is the tip circle radius GEO_TipCircle.polar.len
%	  BT is the Euclidean distance between VAR_TongBlade (B') and tip 
%		circle center CMP_TipCircle.coord.x,y (T)
%	  theta is the included angle QBT
%      and BQ is the blade circle radius (CMP_BladeCircleRad.polar.len)
%
%    theta is the declination of point B' from the horizontal 
%        (CMP_TongBlade.polar.ang)
%	at the tongue ball center (CMP_TongueCircle, C), less the declination 
%	of point T from the horizontal at point B'
%  ============================================================================


% void eval_cmp_BladeCircleRad()
% {
%   double dTxBx,dTyBy, theta, BT, v;
% 
%   dTxBx = CMP_TipCircle.coord.x - VAR_TongBlade.coord.x;
%   dTyBy = CMP_TipCircle.coord.y - VAR_TongBlade.coord.y;
%   BT = sqrt( dTxBx*dTxBx + dTyBy*dTyBy );
%   theta = CMP_TongBlade.polar.ang - (dTxBx?atan2(dTyBy,dTxBx):PIdiv2);
% 
% ***  test for infinite radius  ***
% 
%   v = 2 * (BT * cos(theta) + GEO_TipCircleRad.polar.len);
%   CMP_BladeCircleRad.polar.len = v ? 
%                                  ( BT * BT  - 
%                     GEO_TipCircleRad.polar.len * GEO_TipCircleRad.polar.len
%                                  ) / v : 
%                                  INFINITY;
% 
% }


% dTB = CMP_TipCircle(1:2) - VAR_TongBlade
% BT = norm( dTB )
% if BT == 0
%     theta = CMP_TongBlade(4) - pi/2
% else
%     theta = CMP_TongBlade(4) - atan2(dTB(2),dTB(1))
% end
%  
% v = BT * cos(theta) + GEO_TipCircleRad(3)
% if( v == 0 ) % ***  test for infinite radius  ***
%     CMP_BladeCircleRad(3) = Inf
% else
%     CMP_BladeCircleRad(3) = ( BT^2  - GEO_TipCircleRad(3)^2 ) / 2*v
% end

dTB = CMP_TipCircle(1:2) - VAR_TongBlade
BT = norm( dTB )


theta = CMP_TongBlade(4) - atan2(dTB(2),dTB(1)) 
% if BT==0, arctan2 == 0;
% however, in this case 
% BT*cos( theta) == 0 does not matter what

 
v = BT * cos(theta) + GEO_TipCircleRad(3)
CMP_BladeCircleRad(3) = ( BT^2  - GEO_TipCircleRad(3)^2 ) / 2*v
% CMP_BladeCircleRad(3) == Inf if v==0





%  ============================================================================
%    EVAL_CMP_TONGBLADE (TVV):  angle determining VAR_TongBlade, the
%				point at which the outline diverges from
%				the tongue ball circle. 
%
%    CMP_TongBlade is determined by the tongue tip angle CMP_TipCircle.polar.ang 
%    (THH) plus KEY_BladeOriginOffset.polar.ang, adjusted by a fronting factor 
%    (CMP_FrontFactor.polar.len) proportional to tongue tip length 
%    (CMP_TipCircle.polar.len, STL)
%  ============================================================================

% void eval_cmp_TongBlade()
% {
%   CMP_TongBlade.polar.ang = 0.5 * ( CMP_TipCircle.polar.ang + 
%                                     KEY_BladeOriginOffset.polar.ang + PI ) + 
%                             CMP_FrontFactor.polar.len;
% 
% }

CMP_TongBlade(4) = 0.5 * ( CMP_TipCircle(4) + KEY_BladeOriginOffset(4)+ pi ) + ...
                         CMP_FrontFactor(4)


%  ============================================================================
%    EVAL_CMP_FRONTFACTOR:  tongue tip curve fronting factor
%
%   Proportional to the amount by which tongue tip length 
%   CMP_TipCircle.polar.len (STL)
%   is less than TUN_FrontFactor.polar.len mermels long
%  ============================================================================


% void eval_cmp_FrontFactor()
% {
%   TPARAM *p;
% 
%   p = ( TPARAM * )  & TUN_FrontFactor;	
%   CMP_FrontFactor.polar.len = ( CMP_TipCircle.polar.len - p->osl.offset ) * 
%                               p->osl.scale;
% 
% }

CMP_FrontFactor(3) = ( CMP_TipCircle(3) - TUN_FrontFactor(3) ) * ...
                                                       TUN_FrontFactor(4)
                         


%  ============================================================================
%    EVAL_CMP_FLOOR2TEETH (THV):  angle above jaw angle KEY_JAW (THJ) to
%		    intersection of lower teeth/gum line with floor of mouth
%  ============================================================================


% void eval_cmp_Floor2Teeth()
% {
%   CMP_Floor2Teeth.polar.ang = KEY_Jaw.polar.ang + 
%                               GEO_Teeth2FloorTeeth.polar.ang;
% 
% }

CMP_Floor2Teeth(4) = KEY_Jaw(4) + GEO_Teeth2FloorTeeth(4)


%  ============================================================================
%    EVAL_CMP_TIPCIRCLE (TT, THH, STL)
%
%	CMP_TipCircle.coord.x,y (TT):  center of tongue tip circle
%	CMP_TipCircle.polar.ang (THH): composite tongue tip angle (see below)
%	CMP_TipCircle.polar.len (STL): tongue tip length (see below)
%
%    The center of the tongue tip circle (TT) is determined by polar coords
%    THH and STL offset from CMP_BladeOrigin (B).  STL is constrained in
%    ASY to be the smaller of input length KEY_TipCircle.polar.len (ST) and a
%    maximum length STMX (although this contraint is bypassed when running
%    WAG); here the length is currently just the unconstrained value of
%    KEY_TipCircle.polar.len
%
%    The tongue tip angle THH is the sum of three components:
%      KEY_Jaw.polar.ang (THJ): jaw angle
%      KEY_TipCircle.polar.ang (THT): tip angle
%      CMP_TipTracking.polar.ang (THTC): tongue ball center contribution to 
%        tip angle
%  ============================================================================


% void eval_cmp_TipCircle()
% {
%   CMP_TipCircle.polar.ang = KEY_Jaw.polar.ang + KEY_TipCircle.polar.ang + 
%                             CMP_TipTracking.polar.ang;
%   CMP_TipCircle.polar.len = KEY_TipCircle.polar.len;
%   CMP_TipCircle.coord.x = CMP_BladeOrigin.coord.x + 
%                           CMP_TipCircle.polar.len * 
%                             cos( CMP_TipCircle.polar.ang );
%   CMP_TipCircle.coord.y = CMP_BladeOrigin.coord.y + 
%                           CMP_TipCircle.polar.len * 
%                             sin( CMP_TipCircle.polar.ang );
% 
% }

CMP_TipCircle(3) = KEY_TipCircle(3)
CMP_TipCircle(4) = KEY_Jaw(4) + KEY_TipCircle(4) + CMP_TipTracking(4)

CMP_TipCircle(1:2) = CMP_BladeOrigin(1:2) + CMP_TipCircle(3) * ...
                           [ cos( CMP_TipCircle(4) ) sin( CMP_TipCircle(4) ) ]
 

%  ============================================================================
%    EVAL_CMP_TIPTRACKING (THTC):  tongue ball center contribution to 
%				   tongue tip angle.  Proportional to the
%    difference between KEY_TongueCircle.polar.len (SC) 
%    and TUN_TipTracking.offset
%    Given the current tuning parameters, the most negative tongue center
%    contribution is -27 degrees for SC <= 830, and the least negative
%    contribution is -9 degrees for SC >= 910
%  ============================================================================


% void eval_cmp_TipTracking()
% {
%   TPARAM *p;
%   double len;
% 
%   p = ( TPARAM * )  & TUN_TipTracking;	/* map tuning param args */
%   len = MAX( KEY_TongueCircle.polar.len, p->minx.min );
%   len = MIN( len, p->minx.max );
%   CMP_TipTracking.polar.ang = (len + p->osl.offset) * p->osl.scale;
% 
% }


len = min( max( KEY_TongueCircle(3), TUN_TipTracking(1) ), TUN_TipTracking(2) );
CMP_TipTracking(4) = ( len + TUN_TipTracking(3) ) * TUN_TipTracking(4)

%  ============================================================================
%    EVAL_CMP_ROOT (PHK,THPT):  length & angle of perpendicular bisector of 
%			    tangent VAR_Hyoid (P) : CMP_HyoidTongTan (T)
%  ============================================================================


% void eval_cmp_Root()
% {
%   TPARAM *p;
%   double dx, dy;
% 
%   p = ( TPARAM * )  & TUN_Root;	
% 
%   dx = CMP_HyoidTongTan.coord.x - VAR_Hyoid.coord.x;
%   dy = CMP_HyoidTongTan.coord.y - VAR_Hyoid.coord.y;
% 
%   CMP_Root.polar.ang = dx ? atan2(dy, dx) - PIdiv2 : 0.;
%   CMP_Root.polar.len = (sqrt( dy*dy + dx*dx ) - p->osl.offset) * p->osl.scale;
% 
% }

d(1:2) = CMP_HyoidTongTan(1:2) - VAR_Hyoid(1:2)
CMP_Root(4) = atan2(d(2), d(1)) - pi/2
CMP_Root(3) = ( norm(d) - TUN_Root(3) ) * TUN_Root(4)

%  ============================================================================
%    EVAL_CMP_HYOIDTONGTAN (T):  intersection of tongue body circle with
%				 tangent from VAR_Hyoid (P)
%  ============================================================================


% void eval_cmp_HyoidTongTan()
% {
%   double PT, a, a1, a2, dx, dy;
% 
%   dx = VAR_Hyoid.coord.x - CMP_TongueCircle.coord.x;
%   dy = VAR_Hyoid.coord.y - CMP_TongueCircle.coord.y;
% 
%   PT = dx * dx  +  dy * dy - 
%              KEY_TongueCircleRad.polar.len * KEY_TongueCircleRad.polar.len;
% 
%   % check for error 
%   PT = PT > 0.0       ?  
%          sqrt( PT ) :
%          0.0;          /* ??? plug ??? */
% 
%   a1 = dx ? atan2(dy, dx) : PIdiv2;
%   a2 = KEY_TongueCircleRad.polar.len ? 
%        atan2(PT, KEY_TongueCircleRad.polar.len) : 
%        PIdiv2;
%   a  = a1 - a2;
% 
%   CMP_HyoidTongTan.coord.x = CMP_TongueCircle.coord.x + 
%                              KEY_TongueCircleRad.polar.len * cos( a );
%   CMP_HyoidTongTan.coord.y = CMP_TongueCircle.coord.y + 
%                              KEY_TongueCircleRad.polar.len * sin( a );
% 
% }

d(1:2) = VAR_Hyoid(1:2) - CMP_TongueCircle(1:2)
PT = sqrt( d*d' - KEY_TongueCircleRad(3)^2 )
a = atan2( d(2), d(1) ) - atan2( PT, KEY_TongueCircleRad(3) )
CMP_HyoidTongTan(1:2) = CMP_TongueCircle(1:2) + ...
    KEY_TongueCircleRad(3) * [ cos(a) sin(a) ]



%  ============================================================================
%    EVAL_CMP_TONGUECIRCLE (C):  center of tongue ball circle
%  ============================================================================


% void eval_cmp_TongueCircle()
% {
%   double a;
% 
%   a = KEY_TongueCircle.polar.ang + KEY_Jaw.polar.ang;
%   CMP_TongueCircle.coord.x = GEO_Condyle.coord.x + 
%                              KEY_TongueCircle.polar.len * cos( a );
%   CMP_TongueCircle.coord.y = GEO_Condyle.coord.y + 
%                              KEY_TongueCircle.polar.len * sin( a );
% 
% }

a = KEY_TongueCircle(4) + KEY_Jaw(4)
CMP_TongueCircle(1:2) = GEO_Condyle(1:2) + ...
    KEY_TongueCircle(3) * [ cos(a) sin(a) ]
