% evaluate variable parameters

%  /*
%    ============================================================================
%      CASY_EVAR.C  - evaluate variable parameters

%        Includes:
%  	EVAL_VAR_LARYNX
%  	EVAL_VAR_LAREPI
%  	EVAL_VAR_EPIGLOTTIS
%  	EVAL_VAR_HYOID
%  	EVAL_VAR_ROOT
%  	EVAL_VAR_ROOTTONG
%  	EVAL_VAR_TONGBLADE
%  	EVAL_VAR_BLADETIP
%  	EVAL_VAR_TIPFLOOR
%  	EVAL_VAR_FLOOR
%  	EVAL_VAR_FLOORTEETH
%  	EVAL_VAR_TEETH
%  	EVAL_VAR_TEETHLIP
%  	EVAL_VAR_LIP
%  	EVAL_VAR_TERMINUS
%    ============================================================================
%  */


%  /*
%    ============================================================================
%      EVAL_VAR_LARYNX (1):  anterior edge of larynx

%      X:  related (by tuning function) to movement of VAR_Hyoid
%      Y:  down from VAR_Hyoid by GEO_Hyoid2LarynxY

%      The horizontal position of VAR_Hyoid is compared to a point determined
%      by FIX_PharynxLo.coord.x and TUN_VarLarynxX.minx.max. The scaled 
%      ( TUN_VarLarynxX.osl.scale )
%      difference is used as an offset from the point determined by 
%      FIX_Periarytenoid.coord.x and TUN_VarLarynX.osl.offset.
%    ============================================================================
%  */

%  void eval_var_Larynx() 
%  {
%    TPARAM *p;

%    p = (TPARAM *)&TUN_VarLarynxX;	/* map tuning param args */
%    VAR_Larynx.coord.x = FIX_Periarytenoid.coord.x + p->osl.offset + 
%  	               ( VAR_Hyoid.coord.x - ( FIX_PharynxLo.coord.x + 
%                                                 p->minx.max )
%                         ) * p->osl.scale;
%    VAR_Larynx.coord.y = VAR_Hyoid.coord.y - GEO_Hyoid2LarynxY.polar.len;

%  } /* eval_var_Larynx */


VAR_Larynx(1:2) = [ FIX_Periarytenoid(1) + TUN_VarLarynxX(3) + ...
 	                ( VAR_Hyoid(1) - FIX_PharynxLo(1) - TUN_VarLarynxX(2) ) ...                                                
                          * TUN_VarLarynxX(4) ...
                     VAR_Hyoid(2) - GEO_Hyoid2LarynxY(3) ]

%  /*
%    ============================================================================
%      EVAL_VAR_LAREPI (2):  intersection of epiglottis
%  	and line up and back from front edge of larynx

%      X:  same as epiglottis
%      Y:  down from VAR_Hyoid by GEO_Hyoid2LarEpiY + tuning factor
%  	  determined by horizontal position of VAR_Hyoid

%      Tuning: Adjust vertical position of VAR_LarEpi by scaled 
%      (TUN_LarEpiY.osl.scale)
%      difference between VAR_Hyoid horizontal position and point determined by
%      FIX_PharynxLo and TUN_LarEpiY.osl.offset.
%    ============================================================================
%  */

%  void eval_var_LarEpi() 
%  {
%    TPARAM *p;

%    p = (TPARAM *)&TUN_LarEpiY;		/* map tuning param args */

%    VAR_LarEpi.coord.x = VAR_Epiglottis.coord.x;
%    VAR_LarEpi.coord.y = VAR_Hyoid.coord.y - GEO_Hyoid2LarEpiY.polar.len + 
%  		       ( VAR_Hyoid.coord.x - ( FIX_PharynxLo.coord.x + 
%                                                 p->osl.offset )
%                         ) * p->osl.scale;

%  } /* eval_var_LarEpi */

VAR_LarEpi(1:2) = [ VAR_Epiglottis(1) ...
                    VAR_Hyoid(2) - GEO_Hyoid2LarEpiY(3) + ... 
                    ( VAR_Hyoid(1) - FIX_PharynxLo(1) - TUN_LarEpiY(3) ) ...
                          * TUN_LarEpiY(4) ]


%  /*
%    ============================================================================
%      EVAL_VAR_EPIGLOTTIS (3):  anterior edge (effective top) of epiglottis

%      X:  GEO_Hyoid2Epi left of VAR_Hyoid
%      Y:  same as VAR_Hyoid
%    ============================================================================
%  */

%  void eval_var_Epiglottis() 
%  {

%  /* ***  Compute parameter  *** */
 
%    VAR_Epiglottis.coord.x = VAR_Hyoid.coord.x - GEO_Hyoid2Epi.coord.x;
%    VAR_Epiglottis.coord.y = VAR_Hyoid.coord.y + GEO_Hyoid2Epi.coord.y;

%  } /* eval_var_Epiglottis */


VAR_Epiglottis(1:2) = VAR_Hyoid(1:2) + [ -GEO_Hyoid2Epi(1) GEO_Hyoid2Epi(2) ]

%  /*
%    ============================================================================
%      EVAL_VAR_HYOID (4,P):  anterior end of tongue root at height
%  			     of hyoid (valecula).  
%      X:  KEY_Hyoid
%      Y:  KEY_Hyoid
%    ============================================================================
%  */

%  void eval_var_Hyoid() 
%  {

%    VAR_Hyoid.coord.x = KEY_Hyoid.coord.x;
%    VAR_Hyoid.coord.y = KEY_Hyoid.coord.y;

%  } /* eval_var_Hyoid */


VAR_Hyoid(1:2) = KEY_Hyoid(1:2)

%  /*
%    ============================================================================
%      EVAL_VAR_ROOT (5,K):  end of perpendicular bisector of tangent line
%  			drawn from VAR_Hyoid to CMP_HyoidTongTan (PT)
%      			which is CMP_Root (PHK) plus KEY_RootOffset in length

%      If angle of tangent line - PI/2 CMP_Root.polar.ang (THPT) is in first 
%      quadrant, 
%      the end of its perpendicular bisector will be (below, at, above) tangent 
%      line when tangent line is (<, =, >) 3.5 cm long (384 mermels) (when
%      KEY_RootOffset is 0)
%    ============================================================================
%  */

%  void eval_var_Root() 
%  {
%    double dx, dy, len;

%    dx = CMP_HyoidTongTan.coord.x - VAR_Hyoid.coord.x;
%    dy = CMP_HyoidTongTan.coord.y - VAR_Hyoid.coord.y;

%    len = KEY_RootOffset.polar.len + CMP_Root.polar.len;

%    VAR_Root.coord.x = VAR_Hyoid.coord.x + 0.5 * dx + 
%                       len * cos( CMP_Root.polar.ang );
%    VAR_Root.coord.y = VAR_Hyoid.coord.y + 0.5 * dy + 
%                       len * sin( CMP_Root.polar.ang );

%    /* debug: function isnan is not available under VAX/VMS */
%    /*
%    if( isnan( VAR_Root.coord.x ) ||
%        isnan( VAR_Root.coord.y )
%      )
%    {
%      dx = 0.0;
%    }
%    */

%  } /* eval_var_Root */


VAR_Root(1:2) = (VAR_Hyoid(1:2) + CMP_HyoidTongTan(1:2) ) / 2 + ...
    (KEY_RootOffset(3) + CMP_Root( 3)) * ...
                  [ cos( CMP_Root(4) )   sin( CMP_Root(4) ) ]


%  /*
%    ============================================================================
%      EVAL_VAR_ROOTTONG (6,L):  intersection of tongue body circle with 
%  			        tangent drawn from VAR_Root (K)
%    ============================================================================
%  */

%  void eval_var_RootTong() 
%  {
%    double KL, a, a1, a2, dx, dy;

%    dx = VAR_Root.coord.x - CMP_TongueCircle.coord.x;
%    dy = VAR_Root.coord.y - CMP_TongueCircle.coord.y;

%    KL = dx * dx  +  dy * dy - 
%               KEY_TongueCircleRad.polar.len  *KEY_TongueCircleRad.polar.len;

%    /* check for error */
%    KL = KL > 0.0       ? 
%           sqrt( KL ) : 
%           0.0;             /* ??? plug ??? */

%    a1 = dx ? atan2( dy, dx ) : PIdiv2;
%    a2 = KEY_TongueCircleRad.polar.len ? 
%         atan2( KL, KEY_TongueCircleRad.polar.len ) : 
%         PIdiv2;
%    a  = a1 - a2;

%    VAR_RootTong.coord.x = CMP_TongueCircle.coord.x + 
%                           KEY_TongueCircleRad.polar.len * cos( a );
%    VAR_RootTong.coord.y = CMP_TongueCircle.coord.y + 
%                           KEY_TongueCircleRad.polar.len * sin( a );

%  } /* eval_var_RootTong */


d = VAR_Root(1:2) - CMP_TongueCircle(1:2)
a1 = atan2( d(2), d(1) )

KL = sqrt( max( 0, d(1)^2 + d(2)^2 - KEY_TongueCircleRad(3)^2 ) )
a2 = atan2( KL, KEY_TongueCircleRad(3) )

a = a1 + a2

VAR_RootTong(1:2) = CMP_TongueCircle(1:2) + ...
                    KEY_TongueCircleRad(3) * [ cos(a) sin(a) ]


%  /*
%    ============================================================================
%      EVAL_VAR_TONGBLADE (7, B'):  point of departure of tongue tip curve from
%  			    tongue body circle.  

%      Located at angle CMP_TongBlade.polar.ang from CMP_TongueCircle.coord.x,y  
%    ============================================================================
%  */

%  void eval_var_TongBlade() 
%  {

%    VAR_TongBlade.coord.x = CMP_TongueCircle.coord.x + 
%                            KEY_TongueCircleRad.polar.len * 
%                              cos( CMP_TongBlade.polar.ang );
%    VAR_TongBlade.coord.y = CMP_TongueCircle.coord.y + 
%                            KEY_TongueCircleRad.polar.len * 
%                              sin( CMP_TongBlade.polar.ang );

%  } /* eval_var_TongBlade */

VAR_TongBlade(1:2) = CMP_TongueCircle(1:2) + ...
                       KEY_TongueCircleRad(3) * ...
                             [ cos( CMP_TongBlade(4) ) sin( CMP_TongBlade(4) ) ] 

%  /* 
%    ============================================================================
%      EVAL_VAR_BLADETIP:  point of contact between tongue blade and tongue tip
%  			circles
%    ============================================================================
%  */

%  void eval_var_BladeTip() 
%  {
%    double dx, dy, h; 

%    dx = CMP_TipCircle.coord.x - CMP_BladeCircle.coord.x;
%    dy = CMP_TipCircle.coord.y - CMP_BladeCircle.coord.y;
%    h = sqrt( dx*dx + dy*dy );

%    if (h > 1.e-4 ) {
%      if (CMP_BladeCircleRad.polar.len > 0) {
%        VAR_BladeTip.coord.x = CMP_TipCircle.coord.x - 
%                               dx * GEO_TipCircleRad.polar.len / h;
%        VAR_BladeTip.coord.y = CMP_TipCircle.coord.y - 
%                               dy * GEO_TipCircleRad.polar.len / h;
%      } else {
%        VAR_BladeTip.coord.x = CMP_TipCircle.coord.x + 
%                               dx * GEO_TipCircleRad.polar.len / h;
%        VAR_BladeTip.coord.y = CMP_TipCircle.coord.y + 
%                               dy * GEO_TipCircleRad.polar.len / h;
%      } 
%    } else {
%      VAR_BladeTip.coord.x = CMP_BladeCircle.coord.x;
%      VAR_BladeTip.coord.y = CMP_BladeCircle.coord.y;
%    } 
%  } /* eval_var_BladeTip */


% ??????????? 1-st and 2-nd branches are identical

d = CMP_TipCircle(1:2) - CMP_BladeCircle(1:2)
h = norm( d )
if( h > 1.0e-4 )
    VAR_BladeTip(1:2) = CMP_TipCircle(1:2) - d * GEO_TipCircleRad(3) / h
else
    VAR_BladeTip(1:2) = CMP_BladeCircle(1:2)
end

    
    



%  /* 
%    ============================================================================
%      EVAL_VAR_TIPFLOOR (10):  rightmost tangent of tongue tip circle with line
%  				drawn from VAR_Floor

%      removed from outline if above VAR_BladeTip (on same side of circle)
%    ============================================================================
%  */

%  void eval_var_TipFloor() 
%  {
%    double FT, a, a1, a2, dx, dy;

%    dx = VAR_Floor.coord.x - CMP_TipCircle.coord.x;
%    dy = VAR_Floor.coord.y - CMP_TipCircle.coord.y;
%    dx = fabs(dx);
%    dy = fabs(dy);

  

%    FT = dx*dx + dy*dy - 
%               GEO_TipCircleRad.polar.len * GEO_TipCircleRad.polar.len ;

%    /* checking for error */
%    FT = FT > 0.0       ?
%           sqrt( FT ) :
%           0.0;             /* ??? plug ??? */


%    a1 = dx ? atan2( dy, dx ) : PIdiv2;
%    a2 = GEO_TipCircleRad.polar.len ? 
%         atan2( FT, GEO_TipCircleRad.polar.len ) : 
%         PIdiv2;
%    a  = a1 - a2;

%    VAR_TipFloor.coord.x = CMP_TipCircle.coord.x + 
%                           GEO_TipCircleRad.polar.len * cos( a );
%    VAR_TipFloor.coord.y = CMP_TipCircle.coord.y + 
%                           GEO_TipCircleRad.polar.len * sin( a );


d = abs( VAR_Floor(1:2) - CMP_TipCircle(1:2) )
FT = sqrt( max( 0, d(1)^2+ d(2)^2 - GEO_TipCircleRad(3)^2 ))
a = atan2( d(2), d(1) ) - atan2( FT, GEO_TipCircleRad(3) )
VAR_TipFloor(1:2) = CMP_TipCircle(1:2)  - ...
                          GEO_TipCircleRad(3) * [ cos(a) sin(a) ]


  
    

%    if( VAR_TipFloor.coord.y > VAR_BladeTip.coord.y  && 
%        VAR_BladeTip.coord.x > CMP_TipCircle.coord.x
%      )                                      	/* remove from outline */
%    {
%      SegNODE ** ppSeg;

%      for( ppSeg = & VARSEGS;   * ppSeg;   ppSeg = & (* ppSeg)->next )
%      {
%        if( ( * ppSeg )->segment == & SEG_VAR_BladeTip_TipFloor )
%        {
%          remove_segment( ppSeg );
%          insert_segment( ppSeg, & SEG_VAR_BladeTip_Floor );
%        }
%        else if( ( * ppSeg )->segment == & SEG_VAR_TipFloor_Floor )
%          remove_segment( ppSeg );
%      }

%      VAR_TipFloor.flags |= MASK_SKIP_DRAW;
%    }
%    else
%    {
%      SegNODE ** ppSeg;

%      for( ppSeg = & VARSEGS;   * ppSeg;   ppSeg = & (* ppSeg)->next )
%      {
%        if( ( * ppSeg )->segment == & SEG_VAR_BladeTip_Floor )
%        {
%          remove_segment( ppSeg );
%          insert_segment( ppSeg, & SEG_VAR_TipFloor_Floor );
%          insert_segment( ppSeg, & SEG_VAR_BladeTip_TipFloor );
%        }
%      }
%      VAR_TipFloor.flags &= ~MASK_SKIP_DRAW;
%    }

%    /* debug */
%    /*
%    if( isnan( VAR_TipFloor.coord.x ) ||
%        isnan( VAR_TipFloor.coord.y )
%      )
%    {
%      dx = 0.0;
%    }
%    */

%  } /* eval_var_TipFloor */


%  /* 
%    ============================================================================
%      EVAL_VAR_FLOOR (11):  point defining left-most edge of mouth floor

%      Determined by an offset back from VAR_Teeth (GEO_Teeth2Floor), adjusted 
%      by tongue tip position.  May not lie farther right of VAR_Teeth.  
%      Left of VAR_FloorTeeth vertical position is the same as FloorTeeth; 
%      in the region between VAR_Teeth and VAR_FloorTeeth vertical position 
%      tracks line between them.
%    ============================================================================
%  */

%  void eval_var_Floor() 
%  {
%    TPARAM *p = ( TPARAM * )  & TUN_FloorX;	

%    VAR_Floor.coord.x = VAR_Teeth.coord.x - GEO_Teeth2FloorX.polar.len + 
%                        ( p->osl.offset - 
%                          ( VAR_Teeth.coord.x - CMP_TipCircle.coord.x ) 
%                        ) * p->osl.scale;

VAR_Floor(1) = VAR_Teeth(1) - GEO_Teeth2FloorX(3) + ...
                 ( TUN_FloorX(3) - ( VAR_Teeth(1) - CMP_TipCircle(1) )) ...
                * TUN_FloorX(4)
            
if VAR_Floor(1) >= VAR_Teeth(1) 
  VAR_Floor(1:2) = VAR_Teeth(1:2) 
elseif( VAR_Floor(1) > VAR_FloorTeeth(1) )
  VAR_Floor(2) =  VAR_Teeth(2) ...
                - ( VAR_Teeth(1) - VAR_Floor(1) ) * tan( CMP_Floor2Teeth(4) )
else
  VAR_Floor(2) = VAR_FloorTeeth(2) 
end


%    if( VAR_Floor.coord.x >= VAR_Teeth.coord.x ) 
%                                            /* right of Teeth? clip to Teeth */
%    {	
%      VAR_Floor.coord.x = VAR_Teeth.coord.x;
%      VAR_Floor.coord.y = VAR_Teeth.coord.y;

%      /* remove FloorTeeth from otl */
%      {
%        SegNODE ** ppSeg;
  
%        for( ppSeg = & VARSEGS;   * ppSeg;   ppSeg = & (* ppSeg)->next )
%        {
%  	if( ( * ppSeg )->segment == & SEG_VAR_Floor_FloorTeeth )
%  	{
%  	  remove_segment( ppSeg );
%  	  insert_segment( ppSeg, & SEG_VAR_Floor_Teeth );
%  	}
%  	else if( ( * ppSeg )->segment == & SEG_VAR_FloorTeeth_Teeth )
%  	  remove_segment( ppSeg );
%        }
%      }
%      VAR_FloorTeeth.flags |= MASK_SKIP_DRAW;	
%    } 
%    else if( VAR_Floor.coord.x > VAR_FloorTeeth.coord.x )    
%    /* between Teeth & FloorTeeth */
%    {
%      VAR_Floor.coord.y = VAR_Teeth.coord.y - 
%                          ( VAR_Teeth.coord.x - VAR_Floor.coord.x ) * 
%                          tan( CMP_Floor2Teeth.polar.ang );

%      /* remove FloorTeeth from otl */
%      {
%        SegNODE ** ppSeg;
  
%        for( ppSeg = & VARSEGS;   * ppSeg;   ppSeg = & (* ppSeg)->next )
%        {
%  	if( ( * ppSeg )->segment == & SEG_VAR_Floor_FloorTeeth )
%  	{
%  	  remove_segment( ppSeg );
%  	  insert_segment( ppSeg, & SEG_VAR_Floor_Teeth );
%  	}
%  	else if( ( * ppSeg )->segment == & SEG_VAR_FloorTeeth_Teeth )
%  	  remove_segment( ppSeg );
%        }
%      }
%      VAR_FloorTeeth.flags |= MASK_SKIP_DRAW;	
%    } 
%    else 					/* left of FloorTeeth */
%    {
%      VAR_Floor.coord.y = VAR_FloorTeeth.coord.y;

%      /* include FloorTeeth on otl */
%      {
%        SegNODE ** ppSeg;
  
%        for( ppSeg = & VARSEGS;   * ppSeg;   ppSeg = & (* ppSeg)->next )
%        {
%  	if( ( * ppSeg )->segment == & SEG_VAR_Floor_Teeth )
%  	{
%  	  remove_segment( ppSeg );
%  	  insert_segment( ppSeg, & SEG_VAR_FloorTeeth_Teeth );
%  	  insert_segment( ppSeg, & SEG_VAR_Floor_FloorTeeth );
%  	}
%        }
%      }
%      VAR_FloorTeeth.flags &= ~MASK_SKIP_DRAW;	
%    } 
%  } /* eval_var_Floor */


%  /* 
%    ============================================================================
%      EVAL_VAR_FLOORTEETH (12, V):  intersection of lower teeth/gum line with
%  				   floor of mouth

%      GEO_Floor2Teeth.polar.len back from VAR_Teeth along angle CMP_Floor2Teeth 
%      (THV)
%    ============================================================================
%  */

%  void eval_var_FloorTeeth() 
%  {
%    VAR_FloorTeeth.coord.x = VAR_Teeth.coord.x - 
%                             GEO_Teeth2FloorTeeth.polar.len *
%                               cos(CMP_Floor2Teeth.polar.ang);
%    VAR_FloorTeeth.coord.y = VAR_Teeth.coord.y - 
%                             GEO_Teeth2FloorTeeth.polar.len * 
%                               sin(CMP_Floor2Teeth.polar.ang);
  
%  } /* eval_var_FloorTeeth */



VAR_FloorTeeth(1:2) = VAR_Teeth(1:2) - ...
                       GEO_Teeth2FloorTeeth(3) * ...
                          [ cos( CMP_Floor2Teeth(4) )  sin( CMP_Floor2Teeth(4) ) ]


%  /* 
%    ============================================================================
%      EVAL_VAR_TEETH (13, LJ):  position of lower teeth

%      Distance KEY_Jaw.polar.len (RJ) from GEO_Condyle 
%      at angle KEY_Jaw.polar.ang (THJ)
%    ============================================================================
%  */

%  void eval_var_Teeth() 
%  {

%    VAR_Teeth.coord.x = GEO_Condyle.coord.x + 
%                        KEY_Jaw.polar.len * cos( KEY_Jaw.polar.ang );
%    VAR_Teeth.coord.y = GEO_Condyle.coord.y + 
%                        KEY_Jaw.polar.len * sin( KEY_Jaw.polar.ang );

%  } /* eval_var_Teeth */


VAR_Teeth(1:2) = GEO_Condyle(1:2) + ...
                 KEY_Jaw(3) * [ cos( KEY_Jaw(4) )   sin( KEY_Jaw(4) ) ] 

%  /* 
%    ============================================================================
%      EVAL_VAR_TEETHLIP (14):  intersection of lower lip with lower teeth

%      X: GEO_VarTeeth2TeethLip right of VAR_Teeth
%      Y: VAR_Lip.coord.y
%    ============================================================================
%  */

%  void eval_var_TeethLip() 
%  {

%    VAR_TeethLip.coord.x = VAR_Teeth.coord.x + GEO_VarTeeth2TeethLip.polar.len;
%    VAR_TeethLip.coord.y = VAR_Lip.coord.y;

%  } /* eval_var_TeethLip */

VAR_TeethLip(1:2) = [ VAR_Teeth(1) + GEO_VarTeeth2TeethLip(3)    VAR_Lip(2) ]


%  /* 
%    ============================================================================
%      EVAL_VAR_LIP (15):  lower lip

%      X: KEY_VarLip.coord.x right of VAR_Teeth, plus KEY_VarLip.coord.y 
%              if positive
%      Y: VAR_Teeth offset by KEY_VarLip.coord.y
%    ============================================================================
%  */

%  void eval_var_Lip() 
%  {

%    VAR_Lip.coord.x = VAR_Teeth.coord.x + KEY_VarLip.coord.x;
%    VAR_Lip.coord.y = VAR_Teeth.coord.y + KEY_VarLip.coord.y;
%    if (KEY_VarLip.coord.y > 0.)	/* HLJ > 0 */
%      VAR_TeethLip.coord.x += KEY_VarLip.coord.y;

%  } /* eval_var_Lip */

VAR_Lip(1:2) = VAR_Teeth(1:2) + KEY_VarLip(1:2)
if( KEY_VarLip(2) > 0 )
    VAR_TeethLip(1) = VAR_TeethLip(1) + KEY_VarLip(2)
end
    




%  /* 
%    ============================================================================
%      EVAL_VAR_TERMINUS (16):  tube termination

%      GEO_VarTerminus.polar.len right of VAR_Lip 
%      along angle GEO_VarTerminus.polar.ang
%    ============================================================================
%  */

%  void eval_var_Terminus() 
%  {

%    VAR_Terminus.coord.x = VAR_Lip.coord.x + 
%                           GEO_VarTerminus.polar.len * 
%                             cos( GEO_VarTerminus.polar.ang );
%    VAR_Terminus.coord.y = VAR_Lip.coord.y + 
%                           GEO_VarTerminus.polar.len * 
%                             sin( GEO_VarTerminus.polar.ang );

%  } /* eval_var_Terminus */

VAR_Terminus(1:2) = VAR_Lip(1:2) + ... 
    GEO_VarTerminus(3) * [ cos( GEO_VarTerminus(4) )   sin( GEO_VarTerminus(4) ) ] 
